/*
 * Copyright (c) 2022 Verizon.  Licensed under Verizon EDS SDK Software Licensing Agreement.  See "EDS SDK License Agreement.pdf"
 */

package com.verizon.mec.edsclientsdk.sample;

import android.content.Context;
import android.net.Network;
import android.util.Log;

import com.google.android.gms.common.api.ApiException;
import com.google.android.gms.common.api.CommonStatusCodes;
import com.verizon.mec.edsclientsdk.EdsClientSdkFactory;
import com.verizon.mec.edsclientsdk.api.EdgeSvcNetworkAddrTargetingCriteria;
import com.verizon.mec.edsclientsdk.api.EdgeSvcsTargetingCriteria;
import com.verizon.mec.edsclientsdk.api.EdsConstants;
import com.verizon.mec.edsclientsdk.api.IAsyncOpHandle;
import com.verizon.mec.edsclientsdk.api.IEdgeDiscoveryCallback;
import com.verizon.mec.edsclientsdk.api.IEdsClientSdk;
import com.verizon.mec.edsclientsdk.api.IEdsSdkOptions;
import com.verizon.mec.edsclientsdk.api.IpAddrMode;
import com.verizon.mec.edsclientsdk.data.EdsServiceEndpoint;
import com.verizon.mec.edsclientsdk.data.IEdsSdkVersionInfo;
import com.verizon.mec.edsclientsdk.internal.EdsErrorCode;
import com.verizon.mec.edsclientsdkutils.EdsClientSdkLogWrapper;

public class EdsSdkSample {

    public void example(Context context, Network cellularNetworkConn) {

        // static - enables debug logging for all SDK instances
        EdsClientSdkFactory.setDebugLogging(true);

        // *** Factory class for SDK builder + object creation.
        EdsClientSdkFactory factory = new EdsClientSdkFactory();

        // informational/debugging - get version info -
        IEdsSdkVersionInfo versionInfo = factory.getVersionInfo();
        Log.i("EdsSample", "EdsClientSdk versionName:" + versionInfo.getVersionName() + ", versionCode:" + versionInfo.getVersionCode());


        // *** Create SDK instance using factory builder
        IEdsClientSdk edsSdk = factory.getSdkBuilder(context)
                .setIdmsAttestAuthenticator("someGoogleApiKey", "someMecAppId")
                .build();

        // optionally - for dev/debug wrap SDK in log wrapper (downstream sdk use will transparently log request/response)
        edsSdk = EdsClientSdkLogWrapper.getLogWrappedSdk(edsSdk);

        // SDK Operation Options - Per call optional values, which can be set a default in SDK or supplier per call
        IEdsSdkOptions operationOptions = factory.getDefaultOperationOptions();
        operationOptions.setOperationTimeout(30000);
        operationOptions.setBypassCache(false);
        //operationOptions.setTransportNetwork(cellularNetworkConn);

        // Set default options (used if 'null' options passed to operation), or can be supplied on each call
        edsSdk.setDefaultOperationOptions(operationOptions);

        // *** Perform edge discovery - simple version w/ just endpointIds -
        // - uses UE location w/ default settings (IPv4, default cellular network)
        IAsyncOpHandle opHandle1 = edsSdk.discoverEdgeServices(context, mSdkCallback, "someEdsEndpointsId", cellularNetworkConn, null);

        // Or, Perform edge discovery w/ full targeting criteria + explicit operation options
        EdgeSvcsTargetingCriteria targCriteria = new EdgeSvcNetworkAddrTargetingCriteria("someEdsEndpointsId", cellularNetworkConn, IpAddrMode.PreferIPv6, EdsConstants.REGION_US_EAST1);
        IAsyncOpHandle opHandle2 = edsSdk.discoverEdgeServices(context, mSdkCallback, targCriteria, operationOptions);

        // resulting operation handle allows cancel, preventing callbacks for this operation.
        //opHandle2.cancel();
    }

    IEdgeDiscoveryCallback mSdkCallback = new IEdgeDiscoveryCallback() {
        @Override
        public void onSuccess(EdsServiceEndpoint[] endpoints, String edsJsonResult) {

            Log.i("EdsSample", "onSuccess: " + edsJsonResult);

            // NOTE:  Edge service URLs/IPs will likely NOT be available over WIFI network, if connected.
            // - calling application should likely explicitly target cellular network when interacting
            // with endpoints and should also handle case where cellular connectivity changes.
        }

        @Override
        public void onError(int errorCode, String errorMessage, Throwable ex) {
            Log.w("EdsSample", "onError: " + errorCode + " : " + errorMessage, ex);

            if(errorCode == EdsErrorCode.play_services_unavailable.getValue() ||
               errorCode == EdsErrorCode.play_services_outdated.getValue()) {
                // May prompt user to install / update play-services.
                // or may choose fallback to MEC accessToken authentication
                Log.w("EdsSample", "safetynet is out-of-date on device.  Prompt user to install/upgrade?  fallback to MEC access token?");
            }

            // Safetynet device attestation error.
            // See:  https://developer.android.com/training/safetynet/attestation
            if(errorCode == EdsErrorCode.safetynet_error.getValue()) {

                // Exception may be Google ApiException w/ attestation status details.
                ApiException apiException = (ApiException) ex;
                Log.d("EdsSample", "SafetyNet Error: " +
                        CommonStatusCodes.getStatusCodeString(apiException.getStatusCode()) + ": " + apiException.getStatus());

            }
        }
    };
}
