/*
 * Copyright (c) 2022 Verizon.  Licensed under Verizon EDS SDK Software Licensing Agreement.  See "EDS SDK License Agreement.pdf"
 */

package com.verizon.mec.edsclientsdk.sample;

import android.content.Context;
import android.net.Network;
import android.util.Log;

import com.google.android.gms.tasks.CancellationTokenSource;
import com.google.android.gms.tasks.Task;
import com.google.android.gms.tasks.Tasks;
import com.verizon.mec.edsclientsdk.api.EdgeSvcNetworkAddrTargetingCriteria;
import com.verizon.mec.edsclientsdk.api.EdgeSvcsTargetingCriteria;
import com.verizon.mec.edsclientsdk.api.IEdsClientSdk;
import com.verizon.mec.edsclientsdkutils.EdsClientSdkTaskWrapper;
import com.verizon.mec.edsclientsdkutils.EdsTaskException;
import com.verizon.mec.edsclientsdkutils.data.EdgeDiscoveryResult;

import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

/**
 * The EdsClientSdkTaskWrapper provides Google Task which wraps EDS SDK operations, allowing chaining
 * and blocking execution using Google Tasks capabilities.
 *
 * Success callback results (json string + parsed endpoints objects) are supplied in EdgeDiscoveryResult class.
 *
 * Cancel is supported via Google CancellationToken / CancellationTokenSource.
 *
 * See Google Play Services Tasks API:
 *  https://developers.google.com/android/guides/tasks
 */
public class EdsTaskWrapperSample {

    private final Executor mExecutor = Executors.newSingleThreadExecutor();

    public void taskWrapperExample_Asynchronous(Context context, IEdsClientSdk edsSdk, Network targetCellularNetwork) {

        EdsClientSdkTaskWrapper edsTaskWrapper = new EdsClientSdkTaskWrapper(edsSdk);

        // optional - provide ability to cancel execution.
        CancellationTokenSource cancelSrc = new CancellationTokenSource();

        EdgeSvcsTargetingCriteria targetCriteria = new EdgeSvcNetworkAddrTargetingCriteria("someServiceEndpointId", targetCellularNetwork);
        Task<EdgeDiscoveryResult> edsTask = edsTaskWrapper.discoverEdgeServices(context, targetCriteria, null, cancelSrc.getToken());

        edsTask.addOnSuccessListener(mExecutor, edgeDiscoveryResult -> {
            Log.i("Sample", "Success response: " + edgeDiscoveryResult.getJson());
        }).addOnFailureListener(mExecutor, e -> {
            if(e instanceof EdsTaskException) {
                EdsTaskException edsEx = (EdsTaskException) e;
                Log.w("Sample", "onError - " + edsEx.getEdsErrorCode() + " : " + edsEx.getMessage(), edsEx.getCause());
            } else {
                Log.w("Sample", "onError - exception occurred", e);
            }
        }).addOnCanceledListener(mExecutor, () -> {
            Log.i("Sample", "cancelled via cancelSrc.cancel()");
        });

                // to cancel
                //cancelSrc.cancel();
    }

    public void taskWrapperExample_Synchronous(Context context, IEdsClientSdk edsSdk, Network targetCellularNetwork) {

        EdsClientSdkTaskWrapper edsTaskWrapper = new EdsClientSdkTaskWrapper(edsSdk);

        EdgeSvcsTargetingCriteria targetCriteria = new EdgeSvcNetworkAddrTargetingCriteria("someServiceEndpointId", targetCellularNetwork);
        Task<EdgeDiscoveryResult> edsTask = edsTaskWrapper.discoverEdgeServices(context,
                targetCriteria, null, null);

        // synchronous execution (background thread only!)
        try {
            EdgeDiscoveryResult result = Tasks.await(edsTask, 30, TimeUnit.SECONDS);
            Log.i("Sample", "Successful response: " + result.getJson() );
        }
        catch (ExecutionException e) {
            if(e.getCause() instanceof EdsTaskException) {
                EdsTaskException edsEx = (EdsTaskException) e.getCause();
                Log.w("Sample", "eds error code:" + edsEx.getEdsErrorCode() + ":" + edsEx.getMessage(), edsEx );
            }
            else {
                Log.w("Sample", "exception occurred", e);
            }
        }
        catch (CancellationException e) {
            Log.i("Sample", "task was cancelled via cancellation token");
        }
        catch (InterruptedException e) {
            Log.w("Sample", "task was interrupted.");
        }
        catch (TimeoutException e) {
            Log.w("Sample", "task timeout occurred. (note: timeouts also handled by EDS SDK itself");
        }
    }
}
